==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/av-clamd.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


static char const rcsid[] =
"$Id: av-clamd.c,v 1.8 2004/01/30 15:02:33 lwa Exp $";

#include "config.h"

#include <sys/types.h>


#include <syslog.h>
#include <unistd.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "lmtp.h"

#define SIGNATURE "scanned by ClamAV, http://clamav.sourceforge.net/"

static int av_clamd_scan(struct fstate *state) {
  int fd;
  int stream_fd;
  char inbuf[1000];
  ssize_t inlen;
  unsigned int port;
  char stream[1000];
  char *stream_port;
  fd = rxconnect(&state->av.clamd);
  if (fd == -1)
    return -1;
  if (write(fd, "STREAM\n", sizeof("STREAM\n") -1) == -1 || 
      (inlen = read(fd, inbuf, sizeof(inbuf))) == -1) {
    roll_server(&state->av.clamd, "IO error in Clamd server %s: %s",
		current_server(&state->av.clamd), strerror(errno));
    return -1;
  }
  strcpy(stream, current_server(&state->av.clamd));
  if (*stream == '/') { /* unix socket */
    strcpy(stream, "127.0.0.1:");
  }
  if ((stream_port = strchr(stream, ':')) == NULL)
    return -1;
  stream_port ++;

  if (inlen < sizeof("PORT ") -1 ||
      memcmp("PORT ", inbuf, sizeof("PORT ") -1) != 0) {
    inbuf[inlen] = 0;
    if (inbuf[inlen-1] == '\n')
      inbuf[--inlen] = 0;
    roll_server(&state->av.clamd, "Bad reply in Clamd server %s: %s",
		current_server(&state->av.clamd), inbuf);
    return -1;
  }
  port = strtoul(inbuf + sizeof("PORT ") -1 , NULL, 10);
  sprintf(stream_port, "%d", port);

  if ((stream_fd = xconnect(stream, 1)) == -1) {
    roll_server(&state->av.clamd, "Unable to open Clamd stream socket %s: %s",
                stream, strerror(errno));
    return -1;
  }
#define DUMMY_HEADER "From MAILER-DAEMON Mon Jan  1 00:00:00 1970\n"
  if ((state->av.clamd_mbox_format != 0 &&
       write(stream_fd, DUMMY_HEADER, sizeof(DUMMY_HEADER) -1) == -1) ||
      (mailwrite(state, state->hdr_my_received, HEADER_OTHER, stream_fd)==-1 
       && errno != EPIPE) ||
      close(stream_fd) == -1 ||
      (inlen = read(fd, inbuf, sizeof(inbuf))) == -1) {
    close(stream_fd);
    roll_server(&state->av.clamd, "IO error in Clamd stream %s: %s",
                stream, strerror(errno));
    return -1;
  }
  inbuf[inlen] = 0;
  while(inlen >= 0 && !isgraph((int)inbuf[inlen-1]))
    inbuf[--inlen] = 0;
  if (strncmp(inbuf, "stream: ", sizeof("stream: ") -1) == 0) {
    if (strcmp(inbuf+inlen-sizeof(" OK")+1, " OK") == 0) {
      state->av.desc = strdup(SIGNATURE);
      rxclose(&state->av.clamd);
      return 0;
    }
    if (strcmp(inbuf+inlen-sizeof(" FOUND")+1, " FOUND") == 0) {
      char *virus = inbuf + sizeof("stream: ") -1;
      char descr[AV_DESCR_MAX];
      inbuf[inlen-sizeof(" FOUND")+1] = 0;
      snprintf(descr, AV_DESCR_MAX, "%s\n\tinfected with %s",
	       SIGNATURE, virus);
      descr[sizeof(descr)-1] = 0;
      av_set_virus(state, "ClamAV", virus);
      state->av.desc = strdup(descr);
      rxclose(&state->av.clamd);
      return 1;
    }
  }
  rxclose(&state->av.clamd);
  return -1;
}

av_scanner_t av_clamd_init(struct fstate *state) {
  rxinit(&state->av.clamd, 0, NULL);
  return av_clamd_scan;
}

==> lmtpd-0.10.4.tar.gz: lmtpd-0.10.4/av-clamd.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

static char const rcsid[] UNUSED =
"$Id: av-clamd.c,v 1.9 2004/10/26 09:59:19 lwa Exp $";



#include <sys/types.h>


#include <syslog.h>
#include <unistd.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "lmtp.h"

#define SIGNATURE "scanned by ClamAV, http://clamav.sourceforge.net/"

static int av_clamd_scan(struct fstate *state) {
  int fd;
  int stream_fd;
  char inbuf[1000];
  ssize_t inlen;
  unsigned int port;
  char stream[1000];
  char *stream_port;
  fd = rxconnect(&state->av.clamd);
  if (fd == -1)
    return -1;
  if (write(fd, "STREAM\n", sizeof("STREAM\n") -1) == -1 || 
      (inlen = read(fd, inbuf, sizeof(inbuf))) == -1) {
    roll_server(&state->av.clamd, "IO error in Clamd server %s: %s",
		current_server(&state->av.clamd), strerror(errno));
    return -1;
  }
  strcpy(stream, current_server(&state->av.clamd));
  if (*stream == '/') { /* unix socket */
    strcpy(stream, "127.0.0.1:");
  }
  if ((stream_port = strchr(stream, ':')) == NULL)
    return -1;
  stream_port ++;

  if (inlen < sizeof("PORT ") -1 ||
      memcmp("PORT ", inbuf, sizeof("PORT ") -1) != 0) {
    inbuf[inlen] = 0;
    if (inbuf[inlen-1] == '\n')
      inbuf[--inlen] = 0;
    roll_server(&state->av.clamd, "Bad reply in Clamd server %s: %s",
		current_server(&state->av.clamd), inbuf);
    return -1;
  }
  port = strtoul(inbuf + sizeof("PORT ") -1 , NULL, 10);
  sprintf(stream_port, "%d", port);

  if ((stream_fd = xconnect(stream, 1)) == -1) {
    roll_server(&state->av.clamd, "Unable to open Clamd stream socket %s: %s",
                stream, strerror(errno));
    return -1;
  }
#define DUMMY_HEADER "From MAILER-DAEMON Mon Jan  1 00:00:00 1970\n"
  if ((state->av.clamd_mbox_format != 0 &&
       write(stream_fd, DUMMY_HEADER, sizeof(DUMMY_HEADER) -1) == -1) ||
      (mailwrite(state, state->hdr_my_received, HEADER_OTHER, stream_fd)==-1 
       && errno != EPIPE) ||
      close(stream_fd) == -1 ||
      (inlen = read(fd, inbuf, sizeof(inbuf))) == -1) {
    close(stream_fd);
    roll_server(&state->av.clamd, "IO error in Clamd stream %s: %s",
                stream, strerror(errno));
    return -1;
  }
  inbuf[inlen] = 0;
  while(inlen >= 0 && !isgraph((int)inbuf[inlen-1]))
    inbuf[--inlen] = 0;
  if (strncmp(inbuf, "stream: ", sizeof("stream: ") -1) == 0) {
    if (strcmp(inbuf+inlen-sizeof(" OK")+1, " OK") == 0) {
      state->av.desc = strdup(SIGNATURE);
      rxclose(&state->av.clamd);
      return 0;
    }
    if (strcmp(inbuf+inlen-sizeof(" FOUND")+1, " FOUND") == 0) {
      char *virus = inbuf + sizeof("stream: ") -1;
      char descr[AV_DESCR_MAX];
      inbuf[inlen-sizeof(" FOUND")+1] = 0;
      snprintf(descr, AV_DESCR_MAX, "%s\n\tinfected with %s",
	       SIGNATURE, virus);
      descr[sizeof(descr)-1] = 0;
      av_set_virus(state, "ClamAV", virus);
      state->av.desc = strdup(descr);
      rxclose(&state->av.clamd);
      return 1;
    }
  }
  rxclose(&state->av.clamd);
  return -1;
}

av_scanner_t av_clamd_init(struct fstate *state) {
  rxinit(&state->av.clamd, 0, NULL);
  return av_clamd_scan;
}
